import * as i0 from '@angular/core';
import { computed, effect, untracked, isSignal, isDevMode, inject, DestroyRef, signal, Injector, Injectable, runInInjectionContext, makeEnvironmentProviders, ENVIRONMENT_INITIALIZER } from '@angular/core';
import { Store, createSelectorFactory, defaultMemoize, provideState } from '@ngrx/store';
import { QueryStatus, skipToken, defaultSerializeQueryArgs, fetchBaseQuery as fetchBaseQuery$1, setupListeners, buildCreateApi, coreModule } from '@reduxjs/toolkit/query';
export * from '@reduxjs/toolkit/query';

const UNINITIALIZED_VALUE = Symbol();

function useStableQueryArgs(queryArgs, serialize, endpointDefinition, endpointName) {
    const incoming = computed(() => {
        const incomingArgs = queryArgs();
        return {
            queryArgs: incomingArgs,
            serialized: typeof incomingArgs == 'object'
                ? serialize({ queryArgs: incomingArgs, endpointDefinition, endpointName })
                : incomingArgs,
        };
    });
    let cache = incoming();
    effect(() => {
        if (cache.serialized !== incoming().serialized) {
            cache = incoming();
        }
    });
    return computed(() => (cache.serialized === incoming().serialized ? cache.queryArgs : queryArgs()));
}

function capitalize(str) {
    return str.replace(str[0], str[0].toUpperCase());
}

/**
 * The code in this file is adapted from ngrx/signals
 *
 * ngrx is an open-source project licensed under the MIT license.
 *
 * For more information about the original code, see
 * https://github.com/ngrx/platform
 */
function toDeepSignal(signal) {
    const value = untracked(() => signal());
    if (!isRecord(value)) {
        return signal;
    }
    return new Proxy(signal, {
        get(target, prop) {
            if (!(prop in value)) {
                return target[prop];
            }
            if (!isSignal(target[prop])) {
                Object.defineProperty(target, prop, {
                    value: computed(() => target()[prop]),
                    configurable: true,
                });
            }
            return toDeepSignal(target[prop]);
        },
    });
}
function isRecord(value) {
    return value?.constructor === Object;
}

function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    }
    else {
        return x !== x && y !== y;
    }
}
function shallowEqual(objA, objB) {
    if (is(objA, objB)) {
        return true;
    }
    if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
        return false;
    }
    const keysA = Object.keys(objA);
    const keysB = Object.keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    // eslint-disable-next-line @typescript-eslint/prefer-for-of
    for (let i = 0; i < keysA.length; i++) {
        if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

function safeAssign(target, ...args) {
    return Object.assign(target, ...args);
}

/**
 * Wrapper around `defaultQueryStateSelector` to be used in `useQuery`.
 * We want the initial render to already come back with
 * `{ isUninitialized: false, isFetching: true, isLoading: true }`
 * to prevent that the library user has to do an additional check for `isUninitialized`/
 */
const noPendingQueryStateSelector = (selected) => {
    if (selected.isUninitialized) {
        return {
            ...selected,
            isUninitialized: false,
            isFetching: true,
            isLoading: selected.data !== undefined ? false : true,
            status: QueryStatus.pending,
        };
    }
    return selected;
};
/**
 *
 * @param opts.api - An API with defined endpoints to create hooks for
 * @param opts.moduleOptions.dispatch - The version of the `dispatch` to be used
 * @param opts.moduleOptions.useSelector - The version of the `useSelector` hook to be used
 * @param opts.moduleOptions.getState - The version of the `getState` to be used
 * @returns An object containing functions to generate hooks based on an endpoint
 */
function buildHooks({ api, moduleOptions: { hooks: { dispatch, useSelector, getState }, createSelector, }, serializeQueryArgs, context, }) {
    return { buildQueryHooks, buildMutationHook, usePrefetch };
    function queryStatePreSelector(currentState, lastResult, queryArgs) {
        // if we had a last result and the current result is uninitialized,
        // we might have called `api.util.resetApiState`
        // in this case, reset the hook
        if (lastResult?.endpointName && currentState.isUninitialized) {
            const { endpointName } = lastResult;
            const endpointDefinition = context.endpointDefinitions[endpointName];
            if (serializeQueryArgs({
                queryArgs: lastResult.originalArgs,
                endpointDefinition,
                endpointName,
            }) ===
                serializeQueryArgs({
                    queryArgs,
                    endpointDefinition,
                    endpointName,
                }))
                lastResult = undefined;
        }
        // data is the last known good request result we have tracked
        // or if none has been tracked yet the last good result for the current args
        let data = currentState.isSuccess ? currentState.data : lastResult?.data;
        if (data === undefined)
            data = currentState.data;
        const hasData = data !== undefined;
        // isFetching = true any time a request is in flight
        const isFetching = currentState.isLoading;
        // isLoading = true only when loading while no data is present yet (initial load with no data in the cache)
        const isLoading = !hasData && isFetching;
        // isSuccess = true when data is present
        const isSuccess = currentState.isSuccess || (isFetching && hasData);
        return {
            ...currentState,
            data,
            currentData: currentState.data,
            isFetching,
            isLoading,
            isSuccess,
            // Deep signals required init in undefined atleast
            endpointName: currentState.endpointName,
            error: currentState.error,
            fulfilledTimeStamp: currentState.fulfilledTimeStamp,
            originalArgs: currentState.originalArgs,
            requestId: currentState.requestId,
            startedTimeStamp: currentState.startedTimeStamp,
        };
    }
    function usePrefetch(endpointName, defaultOptions) {
        return (arg, options) => dispatch(api.util.prefetch(endpointName, arg, { ...defaultOptions, ...options }));
    }
    function buildQueryHooks(name) {
        const { initiate, select } = api.endpoints[name];
        const useQuerySubscription = (arg, options = {}) => {
            const subscriptionOptions = computed(() => {
                const { refetchOnReconnect, refetchOnFocus, refetchOnMountOrArgChange, skip = false, pollingInterval = 0, skipPollingIfUnfocused = false, } = typeof options === 'function' ? options() : options;
                return {
                    refetchOnReconnect,
                    refetchOnFocus,
                    refetchOnMountOrArgChange,
                    skip,
                    pollingInterval,
                    skipPollingIfUnfocused,
                };
            });
            const subscriptionArg = computed(() => {
                const subscriptionArg = typeof arg === 'function' ? arg() : arg;
                return subscriptionOptions().skip ? skipToken : subscriptionArg;
            });
            const stableArg = useStableQueryArgs(subscriptionArg, 
            // Even if the user provided a per-endpoint `serializeQueryArgs` with
            // a consistent return value, _here_ we want to use the default behavior
            // so we can tell if _anything_ actually changed. Otherwise, we can end up
            // with a case where the query args did change but the serialization doesn't,
            // and then we never try to initiate a refetch.
            defaultSerializeQueryArgs, context.endpointDefinitions[name], name);
            const stableSubscriptionOptions = computed(() => {
                const { refetchOnReconnect, refetchOnFocus, pollingInterval, skipPollingIfUnfocused } = subscriptionOptions();
                return { refetchOnReconnect, refetchOnFocus, pollingInterval, skipPollingIfUnfocused };
            }, { equal: shallowEqual });
            let subscriptionSelectorsRef;
            if (!subscriptionSelectorsRef) {
                const returnedValue = dispatch(api.internalActions.internal_getRTKQSubscriptions());
                if (isDevMode()) {
                    if (typeof returnedValue !== 'object' || typeof returnedValue?.type === 'string') {
                        throw new Error(`Warning: Middleware for RTK-Query API at reducerPath "${api.reducerPath}" has not been added
              to the store. You must add the middleware for RTK-Query to function correctly!`);
                    }
                }
                subscriptionSelectorsRef = returnedValue;
            }
            let lastRenderHadSubscription = false;
            let promiseRef;
            effect(() => {
                const { queryCacheKey, requestId } = promiseRef || {};
                // HACK We've saved the middleware subscription lookup callbacks into a ref,
                // so we can directly check here if the subscription exists for this query.
                let currentRenderHasSubscription = false;
                if (queryCacheKey && requestId) {
                    currentRenderHasSubscription = !!subscriptionSelectorsRef?.isRequestSubscribed(queryCacheKey, requestId);
                }
                const subscriptionRemoved = !currentRenderHasSubscription && lastRenderHadSubscription;
                lastRenderHadSubscription = currentRenderHasSubscription;
                if (subscriptionRemoved) {
                    promiseRef = undefined;
                }
                const lastPromise = promiseRef;
                if (stableArg() === skipToken) {
                    lastPromise?.unsubscribe();
                    promiseRef = undefined;
                    return;
                }
                const lastSubscriptionOptions = promiseRef?.subscriptionOptions;
                if (!lastPromise || lastPromise.arg !== stableArg()) {
                    lastPromise?.unsubscribe();
                    const promise = dispatch(initiate(stableArg(), {
                        subscriptionOptions: stableSubscriptionOptions(),
                        forceRefetch: subscriptionOptions().refetchOnMountOrArgChange,
                    }));
                    promiseRef = promise;
                }
                else if (stableSubscriptionOptions() !== lastSubscriptionOptions) {
                    lastPromise.updateSubscriptionOptions(stableSubscriptionOptions());
                }
            }, { allowSignalWrites: true });
            inject(DestroyRef).onDestroy(() => {
                promiseRef?.unsubscribe();
                promiseRef = undefined;
            });
            return {
                /**
                 * A method to manually refetch data for the query
                 */
                refetch: () => {
                    if (!promiseRef)
                        throw new Error('Cannot refetch a query that has not been started yet.');
                    return promiseRef.refetch();
                },
            };
        };
        const useLazyQuerySubscription = (options = {}) => {
            const subscriptionArg = signal(UNINITIALIZED_VALUE);
            let promiseRef;
            const stableSubscriptionOptions = computed(() => {
                const { refetchOnReconnect, refetchOnFocus, pollingInterval = 0, skipPollingIfUnfocused = false, } = typeof options === 'function' ? options() : options;
                return { refetchOnReconnect, refetchOnFocus, pollingInterval, skipPollingIfUnfocused };
            }, { equal: shallowEqual });
            effect(() => {
                const lastSubscriptionOptions = promiseRef?.subscriptionOptions;
                if (stableSubscriptionOptions() !== lastSubscriptionOptions) {
                    promiseRef?.updateSubscriptionOptions(stableSubscriptionOptions());
                }
            }, { allowSignalWrites: true });
            let subscriptionOptionsRef = stableSubscriptionOptions();
            effect(() => {
                subscriptionOptionsRef = stableSubscriptionOptions();
            });
            const trigger = (arg, { preferCacheValue = false } = {}) => {
                let promise;
                promiseRef?.unsubscribe();
                promiseRef = promise = dispatch(initiate(arg, { subscriptionOptions: subscriptionOptionsRef, forceRefetch: !preferCacheValue }));
                subscriptionArg.set(arg);
                return promise;
            };
            /* cleanup on unmount */
            inject(DestroyRef).onDestroy(() => {
                promiseRef?.unsubscribe();
            });
            /* if "cleanup on unmount" was triggered from a fast refresh, we want to reinstate the query */
            effect(() => {
                if (subscriptionArg() !== UNINITIALIZED_VALUE && !promiseRef) {
                    trigger(subscriptionArg(), { preferCacheValue: true });
                }
            });
            const lastArg = computed(() => (subscriptionArg() !== UNINITIALIZED_VALUE ? subscriptionArg() : skipToken), {
                equal: shallowEqual,
            });
            return [trigger, lastArg];
        };
        const useQueryState = (arg, options = {}) => {
            const stateOptions = computed(() => {
                const { skip = false, selectFromResult } = typeof options === 'function' ? options() : options;
                return { skip, selectFromResult };
            });
            const subscriptionArg = computed(() => {
                const subscriptionArg = typeof arg === 'function' ? arg() : arg;
                return stateOptions().skip ? skipToken : subscriptionArg;
            });
            const stableArg = useStableQueryArgs(subscriptionArg, serializeQueryArgs, context.endpointDefinitions[name], name);
            let lastValue;
            const currentState = computed(() => {
                const selectDefaultResult = createSelector(select(stableArg()), (subState) => queryStatePreSelector(subState, lastValue, stableArg()));
                const { selectFromResult } = stateOptions();
                const querySelector = selectFromResult
                    ? createSelector(selectDefaultResult, selectFromResult)
                    : selectDefaultResult;
                const currentState = useSelector((state) => querySelector(state), {
                    equal: shallowEqual,
                });
                lastValue = selectDefaultResult(getState());
                return currentState();
            });
            const deepSignal = toDeepSignal(currentState);
            return deepSignal;
        };
        return {
            useQueryState,
            useQuerySubscription,
            useLazyQuerySubscription,
            useLazyQuery(options) {
                const [trigger, arg] = useLazyQuerySubscription(options);
                const subscriptionOptions = computed(() => ({
                    ...options,
                    skip: arg() === UNINITIALIZED_VALUE,
                }));
                const queryStateResults = useQueryState(arg, subscriptionOptions);
                Object.assign(queryStateResults, { fetch: trigger });
                Object.assign(queryStateResults, { lastArg: arg });
                return queryStateResults;
            },
            useQuery(arg, options) {
                const querySubscriptionResults = useQuerySubscription(arg, options);
                const subscriptionOptions = computed(() => {
                    const subscriptionArg = typeof arg === 'function' ? arg() : arg;
                    const { skip } = typeof options === 'function' ? options() : options || {};
                    const selectFromResult = subscriptionArg === skipToken || skip ? undefined : noPendingQueryStateSelector;
                    return { selectFromResult, ...options };
                });
                const queryStateResults = useQueryState(arg, subscriptionOptions);
                Object.assign(queryStateResults, querySubscriptionResults);
                return queryStateResults;
            },
            selector: select,
        };
    }
    function buildMutationHook(name) {
        const { initiate, select } = api.endpoints[name];
        const useMutation = ({ selectFromResult, fixedCacheKey } = {}) => {
            const promiseRef = signal(undefined);
            effect((onCleanup) => {
                const currentPromise = promiseRef();
                onCleanup(() => {
                    if (!currentPromise?.arg.fixedCacheKey) {
                        untracked(() => currentPromise?.reset());
                    }
                });
            });
            const triggerMutation = (arg) => {
                const promise = dispatch(initiate(arg, { fixedCacheKey }));
                promiseRef.set(promise);
                return promise;
            };
            const fixedSelect = (args) => (state) => {
                const currentState = select(args)(state);
                return {
                    ...currentState,
                    // Deep signals required init in undefined atleast
                    data: currentState.data,
                    endpointName: currentState.endpointName,
                    error: currentState.error,
                    fulfilledTimeStamp: currentState.fulfilledTimeStamp,
                    requestId: currentState.requestId,
                    startedTimeStamp: currentState.startedTimeStamp,
                };
            };
            const requestId = computed(() => promiseRef()?.requestId);
            const selectDefaultResult = (requestId) => fixedSelect({ fixedCacheKey, requestId });
            const mutationSelector = (requestId) => selectFromResult
                ? createSelector(selectDefaultResult(requestId), selectFromResult)
                : selectDefaultResult(requestId);
            const currentState = computed(() => useSelector(mutationSelector(requestId()), { equal: shallowEqual }));
            const originalArgs = computed(() => (fixedCacheKey == null ? promiseRef()?.arg.originalArgs : undefined));
            const reset = () => {
                if (promiseRef()) {
                    promiseRef.set(undefined);
                }
                if (fixedCacheKey) {
                    dispatch(api.internalActions.removeMutationResult({
                        requestId: requestId(),
                        fixedCacheKey,
                    }));
                }
            };
            const finalState = computed(() => currentState()());
            const deepSignal = toDeepSignal(finalState);
            Object.assign(deepSignal, { dispatch: triggerMutation });
            Object.assign(deepSignal, { originalArgs });
            Object.assign(deepSignal, { reset });
            return deepSignal;
        };
        return {
            useMutation,
            selector: select,
        };
    }
}

let service;
let injector;
function dispatch(action) {
    service?.dispatch(action);
    return action;
}
function getState() {
    return service?.getState();
}
function select(mapFn, options) {
    return service?.select(mapFn, options);
}
class ThunkService {
    constructor() {
        this.#injector = inject(Injector);
        this.#store = inject(Store);
        this.getState = this.#store.selectSignal((state) => state);
    }
    #injector;
    #store;
    init() {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        service = this;
        injector = this.#injector;
    }
    dispatch(action) {
        this.#store.dispatch(action);
    }
    select(mapFn, options) {
        return this.#store.selectSignal(mapFn, options);
    }
    /** @nocollapse */ static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.2.2", ngImport: i0, type: ThunkService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    /** @nocollapse */ static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.2.2", ngImport: i0, type: ThunkService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.2.2", ngImport: i0, type: ThunkService, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }] });

function isQueryDefinition(e) {
    return e.type === 'query';
}
function isMutationDefinition(e) {
    return e.type === 'mutation';
}

const angularHooksModuleName = /* @__PURE__ */ Symbol();
const _createSelector = createSelectorFactory((projector) => defaultMemoize(projector, shallowEqual, shallowEqual));
/**
 * Creates a module that generates angular hooks from endpoints, for use with `buildCreateApi`.
 *
 *  @example
 * ```ts
 * const customCreateApi = buildCreateApi(
 *   coreModule(),
 *   angularHooksModule({
 *     hooks: {
 *       useDispatch: createDispatchHook(MyContext),
 *       useSelector: createSelectorHook(MyContext),
 *       useStore: createStoreHook(MyContext)
 *     }
 *   })
 * );
 * ```
 *
 * @returns A module for use with `buildCreateApi`
 */
const angularHooksModule = ({ hooks = {
    dispatch: dispatch,
    useSelector: select,
    getState: getState,
}, createSelector = _createSelector, ...rest } = {}) => {
    if (isDevMode()) {
        const hookNames = ['dispatch', 'useSelector', 'getState'];
        let warned = false;
        for (const hookName of hookNames) {
            // warn for old hook options
            if (Object.keys(rest).length > 0) {
                if (rest[hookName]) {
                    if (!warned) {
                        console.warn('As of RTK 2.0, the hooks now need to be specified as one object, provided under a `hooks` key:' +
                            '\n`angularHooksModule({ hooks: { dispatch, useSelector, getState } })`');
                        warned = true;
                    }
                }
                // @ts-expect-error migrate
                hooks[hookName] = rest[hookName];
            }
            // then make sure we have them all
            if (typeof hooks[hookName] !== 'function') {
                throw new Error(`When using custom hooks for context, all ${hookNames.length} hooks need to be provided: ${hookNames.join(', ')}.\nHook ${hookName} was either not provided or not a function.`);
            }
        }
    }
    return {
        name: angularHooksModuleName,
        init(api, { serializeQueryArgs }, context) {
            const anyApi = api;
            const { buildQueryHooks, buildMutationHook, usePrefetch } = buildHooks({
                api,
                moduleOptions: { hooks, createSelector },
                serializeQueryArgs,
                context,
            });
            safeAssign(anyApi, { usePrefetch });
            safeAssign(anyApi, { dispatch: hooks.dispatch });
            return {
                injectEndpoint(endpointName, definition) {
                    if (isQueryDefinition(definition)) {
                        const { useQuery, useLazyQuery, useLazyQuerySubscription, useQueryState, useQuerySubscription, selector } = buildQueryHooks(endpointName);
                        safeAssign(anyApi.endpoints[endpointName], {
                            useQuery,
                            useLazyQuery,
                            useLazyQuerySubscription,
                            useQueryState,
                            useQuerySubscription,
                            selector,
                        });
                        api[`use${capitalize(endpointName)}Query`] = useQuery;
                        api[`useLazy${capitalize(endpointName)}Query`] = useLazyQuery;
                    }
                    else if (isMutationDefinition(definition)) {
                        const { useMutation, selector } = buildMutationHook(endpointName);
                        safeAssign(anyApi.endpoints[endpointName], { useMutation, selector });
                        api[`use${capitalize(endpointName)}Mutation`] = useMutation;
                    }
                },
            };
        },
    };
};

function fetchBaseQuery(paramsOrFactory = {}) {
    if (typeof paramsOrFactory === 'object')
        return fetchBaseQuery$1(paramsOrFactory);
    return async (args, api, extraOptions) => {
        const baseQuery = runInInjectionContext(injector, paramsOrFactory);
        return await baseQuery(args, api, extraOptions);
    };
}

function provideStoreApi(api, { setupListeners: setupListeners$1 } = {}) {
    setupListeners$1 === false ? undefined : setupListeners(api.dispatch, setupListeners$1);
    return makeEnvironmentProviders([
        {
            provide: ENVIRONMENT_INITIALIZER,
            multi: true,
            useValue() {
                inject(ThunkService).init();
            },
        },
        provideState(api.reducerPath, api.reducer),
    ]);
}

const createApi = (options) => {
    const reducerPath = options.reducerPath;
    const getState$1 = () => {
        const storeState = getState();
        return storeState?.[reducerPath]
            ? storeState
            : // Query inside forFeature (Code splitting)
                { [reducerPath]: storeState };
    };
    const next = (action) => {
        if (typeof action === 'function') {
            return action(dispatch$1, getState$1, {});
        }
        return dispatch(action);
    };
    const dispatch$1 = (action) => middleware(next)(action);
    const createApi = /* @__PURE__ */ buildCreateApi(coreModule(), angularHooksModule({
        hooks: {
            dispatch: dispatch$1,
            getState: getState$1,
            useSelector: select,
        },
    }));
    const api = createApi(options);
    const middleware = api.middleware({ dispatch: dispatch$1, getState: getState$1 });
    return api;
};

/**
 * Generated bundle index. Do not edit.
 */

export { angularHooksModule, angularHooksModuleName, createApi, fetchBaseQuery, provideStoreApi };
//# sourceMappingURL=ngrx-rtk-query.mjs.map
